@@ -1,7 +1,6 @@
 use crate::build_context::BridgeModel;
 use crate::python_interpreter::InterpreterKind;
-use crate::zig::prepare_zig_linker;
-use crate::{BuildContext, PythonInterpreter};
+use crate::{BuildContext, PlatformTag, PythonInterpreter};
 use anyhow::{anyhow, bail, Context, Result};
 use fat_macho::FatWriter;
 use fs_err::{self as fs, File};
@@ -9,7 +8,7 @@ use std::collections::HashMap;
 use std::env;
 use std::io::{BufReader, Read};
 use std::path::{Path, PathBuf};
-use std::process::{Command, Stdio};
+use std::process::Stdio;
 use std::str;
 
 /// Builds the rust crate into a native module (i.e. an .so or .dll) for a
@@ -106,17 +105,12 @@ fn compile_target(
     python_interpreter: Option<&PythonInterpreter>,
     bindings_crate: &BridgeModel,
 ) -> Result<HashMap<String, PathBuf>> {
-    let (zig_cc, zig_cxx) = if context.zig && context.target.is_linux() {
-        let (cc, cxx) =
-            prepare_zig_linker(context).context("Failed to create zig linker wrapper")?;
-        (Some(cc), Some(cxx))
-    } else {
-        (None, None)
-    };
-
-    let mut shared_args = vec!["--manifest-path", context.manifest_path.to_str().unwrap()];
-
-    shared_args.extend(context.cargo_extra_args.iter().map(String::as_str));
+    let target = &context.target;
+    let mut shared_args: Vec<_> = context
+        .cargo_extra_args
+        .iter()
+        .map(String::as_str)
+        .collect();
 
     if context.release {
         let has_cargo_profile = shared_args
@@ -139,7 +133,7 @@ fn compile_target(
     // TODO: What do we do when there are multiple bin targets?
     match bindings_crate {
         BridgeModel::Bin => shared_args.push("--bins"),
-        BridgeModel::Cffi | BridgeModel::Bindings(_) | BridgeModel::BindingsAbi3(_, _) => {
+        BridgeModel::Cffi | BridgeModel::Bindings(..) | BridgeModel::BindingsAbi3(..) => {
             shared_args.push("--lib");
             // https://github.com/rust-lang/rust/issues/59302#issue-422994250
             // We must only do this for libraries as it breaks binaries
@@ -164,8 +158,8 @@ fn compile_target(
     let macos_dylib_install_name = format!("link-args=-Wl,-install_name,@rpath/{}", so_filename);
 
     // https://github.com/PyO3/pyo3/issues/88#issuecomment-337744403
-    if context.target.is_macos() {
-        if let BridgeModel::Bindings(_) | BridgeModel::BindingsAbi3(_, _) = bindings_crate {
+    if target.is_macos() {
+        if let BridgeModel::Bindings(..) | BridgeModel::BindingsAbi3(..) = bindings_crate {
             let mac_args = &[
                 "-C",
                 "link-arg=-undefined",
@@ -191,7 +185,7 @@ fn compile_target(
         // however, we get an exit code 0xc0000005 if we try the same with
         // `/FORCE:UNDEFINED`, so we still look up the python interpreter
         // and pass the location of the lib with the definitions.
-        if context.target.is_windows() {
+        if target.is_windows() {
             let python_interpreter = python_interpreter
                 .expect("Must have a python interpreter for building abi3 on windows");
             pythonxy_lib_folder = format!("native={}", python_interpreter.libs_dir.display());
@@ -199,7 +193,7 @@ fn compile_target(
         }
     }
 
-    let cargo_args = vec!["rustc", "--message-format", "json"];
+    let cargo_args = vec!["--message-format", "json"];
 
     let build_args: Vec<_> = cargo_args
         .iter()
@@ -207,12 +201,30 @@ fn compile_target(
         .chain(&["--"])
         .chain(&rustc_args)
         .collect();
-    let command_str = build_args
-        .iter()
-        .fold("cargo".to_string(), |acc, x| acc + " " + x);
 
-    let mut let_binding = Command::new("cargo");
-    let build_command = let_binding
+    let mut build = cargo_zigbuild::Build {
+        manifest_path: Some(context.manifest_path.clone()),
+        ..Default::default()
+    };
+    let target_triple = target.target_triple();
+    if !context.zig {
+        build.disable_zig_linker = true;
+        if target.user_specified {
+            build.target = Some(target_triple.to_string());
+        }
+    } else {
+        let zig_triple = if target.is_linux() {
+            match context.platform_tag {
+                Some(PlatformTag::Manylinux { x, y }) => format!("{}.{}.{}", target_triple, x, y),
+                _ => target_triple.to_string(),
+            }
+        } else {
+            target_triple.to_string()
+        };
+        build.target = Some(zig_triple);
+    }
+    let mut build_command = build.build_command("rustc")?;
+    build_command
         .env("RUSTFLAGS", rust_flags)
         .args(&build_args)
         // We need to capture the json messages
@@ -221,20 +233,6 @@ fn compile_target(
         // but forwarding stderr is still useful in case there some non-json error
         .stderr(Stdio::inherit());
 
-    // Also set TARGET_CC and TARGET_CXX for cc-rs and cmake-rs
-    if let Some(zig_cc) = zig_cc {
-        let env_target = context
-            .target
-            .target_triple()
-            .to_uppercase()
-            .replace("-", "_");
-        build_command.env("TARGET_CC", &zig_cc);
-        build_command.env(format!("CARGO_TARGET_{}_LINKER", env_target), &zig_cc);
-    }
-    if let Some(zig_cxx) = zig_cxx {
-        build_command.env("TARGET_CXX", &zig_cxx);
-    }
-
     if let BridgeModel::BindingsAbi3(_, _) = bindings_crate {
         let is_pypy = python_interpreter
             .map(|p| p.interpreter_kind == InterpreterKind::PyPy)
@@ -249,7 +247,7 @@ fn compile_target(
     if let Some(python_interpreter) = python_interpreter {
         // Target python interpreter isn't runnable when cross compiling
         if python_interpreter.runnable {
-            if bindings_crate.is_bindings("pyo3") {
+            if bindings_crate.is_bindings("pyo3") || bindings_crate.is_bindings("pyo3-ffi") {
                 build_command.env("PYO3_PYTHON", &python_interpreter.executable);
             }
 
@@ -314,9 +312,15 @@ fn compile_target(
         .expect("Failed to wait on cargo child process");
 
     if !status.success() {
+        let command_str = build_args
+            .iter()
+            .map(ToString::to_string)
+            .collect::<Vec<String>>()
+            .join(" ");
         bail!(
-            r#"Cargo build finished with "{}": `{}`"#,
+            r#"Cargo build finished with "{}": `cargo rustc --manifest-path {} {}`"#,
             status,
+            context.manifest_path.display(),
             command_str
         )
     }
